@@ -4,144 +4,94 @@ use strict;
 use warnings;
 
 use Carp qw(croak);
-use Scalar::Util qw(weaken isweak);
+use Mock::Sub::Child;
+use Scalar::Util qw(weaken);
 
-our $VERSION = '1.01';
+our $VERSION = '1.03';
 
 sub new {
-    return bless {}, shift;
-}
-sub mock {
-
-    my $thing = shift;
-    my $sub = shift;
-    my $self;
-
-    if (ref($thing) eq __PACKAGE__ && $thing->{unmocked}){
-        $self = $thing;
-        $self->{unmocked} = 0;
-    }
-    else {
-        $self = bless {}, __PACKAGE__;
-        if (! defined wantarray){
-            croak "\n\ncalling mock() in void context isn't allowed. ";
-        }
-    }
-
+    my $self = bless {}, shift;
     %{ $self } = @_;
-
-    $sub = "main::$sub" if $sub !~ /::/;
-
-    my $fake;
-
-    if (! exists &$sub){
-        $fake = 1;
-        warn "\n\nWARNING!: we've mocked a non-existent subroutine. " .
-             "the specified sub does not exist.\n\n";
-    }
-
-    $self->_check_side_effect($self->{side_effect});
-    push @{ $self->{return} }, $self->{return_value};
-
-    $self->{name} = $sub;
-    $self->{orig} = \&$sub if ! $fake;
-
-    my $called;
-
-    {
-        no strict 'refs';
-        no warnings 'redefine';
-
-        *$sub = sub {
-
-            weaken $self if ! isweak $self;
-
-            @{ $self->{called_with} } = @_;
-            $self->{called_count} = ++$called;
-
-            if ($self->{side_effect}) {
-                if (wantarray){
-                    my @effect = $self->{side_effect}->(@_);
-                    return @effect if @effect;
-                }
-                else {
-                    my $effect = $self->{side_effect}->(@_);
-                    return $effect if defined $effect;
-                }
-            }
-
-            return if ! $self->{return};
-
-            return ! wantarray && @{ $self->{return} } == 1
-                ? $self->{return}[0]
-                : @{ $self->{return} };
-        };
-    }
     return $self;
 }
-sub unmock {
+sub mock {
     my $self = shift;
-    my $sub = $self->{name};
+    my $sub = shift;
 
-    $self->{unmocked} = 1;
+    if (ref($self) ne 'Mock::Sub'){
+        croak
+            "calling mock() on the Mock::Sub class is no longer permitted. " .
+            "create a new mock object with Mock::Sub->new;, then call mock " .
+            "with my \$sub_object = \$mock->mock('sub_name'); ";
+    }
+    my %p = @_;
+    for (keys %p){
+        $self->{$_} = $p{$_};
+    }
 
-    {
-        no strict 'refs';
-        no warnings 'redefine';
+    if (! defined wantarray){
+        croak "\n\ncalling mock() in void context isn't allowed. ";
+    }
 
-        if (defined $self->{orig}) {
-            *$sub = \&{ $self->{orig} };
-        }
-        else {
-            undef *$sub if $self->{name};
+    my $child = Mock::Sub::Child->new;
+
+    $child->side_effect($self->{side_effect});
+    $child->return_value($self->{return_value});
+
+    $self->{objects}{$sub}{obj} = $child;
+    $child->mock($sub);
+
+    # remove the REFCNT to the child, or else DESTROY won't be called
+    weaken $self->{objects}{$sub}{obj};
+
+    return $child;
+}
+sub mocked_subs {
+    my $self = shift;
+
+    my @names;
+
+    for (keys %{ $self->{objects} }) {
+        if ($self->mocked_state($_)){
+            push @names, $_;
         }
     }
-    $self->reset;
+    return @names;
 }
-sub called {
-    return shift->called_count ? 1 : 0;
-}
-sub called_count {
-    return shift->{called_count};
-}
-sub called_with {
+sub mocked_objects {
     my $self = shift;
-    if (! $self->called){
-        croak "\n\ncan't call called_with() before the mocked sub has " .
-            "been called. ";
+
+    my @mocked;
+    for (keys %{ $self->{mocked} }){
+        push @mocked, $self->{mocked}{$_}{object};
     }
-    return @{ $self->{called_with} };
+    return @mocked;
 }
-sub name {
-    return shift->{name};  
-}
-sub reset {
-    for (qw(side_effect return_value return called called_count called_with)){
-        delete $_[0]->{$_};
+sub mocked_state {
+    my ($self, $sub) = @_;
+
+    if (! $sub){
+        croak "calling mocked_state() on a Mock::Sub object requires a sub " .
+              "name to be passed in as its only parameter. ";
     }
-}
-sub return_value {
-    my $self = shift;
-    @{ $self->{return} } = @_;
-}
-sub side_effect {
-    $_[0]->_check_side_effect($_[1]);
-    $_[0]->{side_effect} = $_[1];
-}
-sub _check_side_effect {
-    if (defined $_[1] && ref $_[1] ne 'CODE') {
-        croak "\n\nside_effect parameter must be a code reference. ";
+
+    eval {
+        my $test = $self->{objects}{$sub}{obj}->mocked_state();
+    };
+    if ($@){
+        croak "can't call mocked_state() on the class if the sub hasn't yet " .
+              "been mocked. ";
     }
+    return $self->{objects}{$sub}{obj}->mocked_state;
 }
 sub DESTROY {
-    $_[0]->unmock;
 }
-sub _end {}; # vim fold placeholder
+sub __end {}; # vim fold placeholder
 
 1;
 =head1 NAME
 
-Mock::Sub - Mock module, package, object and standard subroutines, with unit testing in mind.
+Mock::Sub - Mock package, object and standard subroutines, with unit testing in mind.
 
 
 =head1 SYNOPSIS
@@ -150,7 +100,15 @@ Mock::Sub - Mock module, package, object and standard subroutines, with unit tes
 
     use Mock::Sub;
 
-    my $foo = Mock::Sub->mock('Package::foo');
+    # create the parent mock object
+
+    my $mock = Mock::Sub->new;
+
+    # mock some subs... all of the following are children under the
+    # umbrella of the above $mock object we created
+
+    my $foo = $mock->mock('Package::foo');
+    my $bar = $mock->mock('Package::bar');
 
     # wait until the mocked sub is called
 
@@ -163,25 +121,21 @@ Mock::Sub - Mock module, package, object and standard subroutines, with unit tes
     $foo->called_count; # how many times was it called?
     $foo->called_with;  # array of params sent to sub
 
-    # create a mock object to reduce typing when multiple subs are mocked
-
-    my $mock = Mock::Sub->new;
-
-    my $foo = $mock->mock('Package::foo');
-    my $bar = $mock->mock('Package::bar');
-
-    # have the mocked sub return something when it's called (list or scalar)
+    # have the mocked sub return something when it's called (list or scalar).
+    # See new() to find out how to set a return value once and have it used in
+    # all child mocks
 
     $foo->return_value(1, 2, {a => 1});
     my @return = Package::foo;
 
     # have the mocked sub perform an action (the side effect function receives
-    # the parameters sent into the mocked sub)
+    # the parameters sent into the mocked sub). See new() to find out how to
+    # set side_effect up once, and have it copied to all child mocks
 
-    $foo->side_effect( sub { die "eval catch"; } );
+    $foo->side_effect( sub { die "eval catch" if @_; } );
 
-    eval { Package::foo; };
-    like ($@, qr/eval catch/, "side_effect worked");
+    eval { Package::foo(1); };
+    like ($@, qr/eval catch/, "side_effect worked with params");
 
     # extract the parameters the sub was called with (best if you know what
     # the original sub is expecting)
@@ -199,9 +153,29 @@ Mock::Sub - Mock module, package, object and standard subroutines, with unit tes
     $foo->unmock;
 
     # re-mock a sub using the same object after unmocking (this is the only
-    # time void context with mock() is permitted)
+    # time void context with mock() is permitted). Note that child mocks don't
+    # take a sub parameter to mock()
 
-    $foo->mock('One::foo');
+    $foo->mock;
+
+    # check if a sub is mocked
+
+    my $state = $foo->state;
+
+    # mock out a CORE:: function. Be warned that this *must* be done within
+    # compile stage (BEGIN), and the function can NOT be unmocked prior
+    # to the completion of program execution
+
+    my ($mock, $caller);
+
+    BEGIN {
+        $mock = Mock::Sub->new;
+        # core function caller() is now mocked permanently after the following line
+        $caller = $mock->mock('caller');
+    };
+
+    $caller->return_value(55);
+    caller(); # mocked caller() called
 
 =head1 DESCRIPTION
 
@@ -263,38 +237,74 @@ MyModule::first).
     test(2);
     is ($second->called, 1, "2nd if() statement covered");
 
+=head1 MOCK OBJECT METHODS
 
-=head1 METHODS
+=head2 C<new(%opts)>
 
-=head2 C<new>
+Instantiates and returns a new C<Mock::Sub> object, ready to be used to start
+cteating mocked sub objects.
 
-Instantiates and returns a new C<Mock::Sub> object.
-
-=head2 C<mock('sub', %opts)>
-
-Instantiates a new object on each call (calling in void context is not
-allowed). 'sub' is the name of the subroutine to mock (requires full package
-name if the sub isn't in C<main::>).
-
-The mocked sub will return undef if a return value isn't set, or a side effect
-doesn't return anything.
-
-Options:
+Optional options:
 
 =over 4
 
 =item C<return_value>
 
-Set this to have the mocked sub return anything you wish (accepts a single
-scalar only. See C<return_value()> method to return a list and for further
-information).
+Set this to have all mocked subs created with this mock object return anything
+you wish (accepts a single scalar only. See C<return_value()> method to return
+a list and for further information). You can also set it in individual mocks
+only (see C<mock()>).
 
 =item C<side_effect>
 
-See C<side_effect()> method.
+Set this in C<new()> to have the side effect passed into all child mocks
+created with this object. See C<side_effect()> method.
 
 =back
 
+=head2 C<mock('sub', %opts)>
+
+Instantiates and returns a new mock object on each call (calling in
+void context is not permitted). 'sub' is the name of the subroutine to mock
+(requires full package name if the sub isn't in C<main::>.
+
+The mocked sub will return undef if a return value isn't set, or a side effect
+doesn't return anything.
+
+Optional options:
+
+Both C<return_value> and C<side_effect> can be set in this method to
+individualize each mock object. Set in C<new> to have all mock objects use
+the same configuration.
+
+There's also C<return_value()> and C<side_effect()> methods if you want to
+set, change or remove these values after instantiation.
+
+=head2 mocked_subs
+
+Returns a list of all subs that are currently mocked under the parent mock
+object.
+
+=head2 mocked_objects
+
+Returns a list of all sub objects underneath the parent mock object, regardless
+if its sub is currently mocked or not.
+
+=head2 mocked_state('Sub::Name')
+
+Returns whether a sub currently under the parent mock object is mocked or not.
+Croaks if there is no object with the sub name parameter.
+
+=head1 SUB OBJECT METHODS
+
+These methods are for the children mocked sub objects returned from the
+parent mock object. See L<MOCK OBJECT METHODS> for methods related
+to the parent mock object.
+
+=head2 C<mock>
+
+Re-mocks the sub within the object after calling C<unmock> on it.
+
 =head2 C<unmock>
 
 Restores the original functionality back to the sub, and runs C<reset()> on
@@ -313,6 +323,11 @@ Returns the number of times the mocked sub has been called.
 Returns an array of the parameters sent to the subroutine. C<croak()s> if
 we're called before the mocked sub has been called.
 
+=head2 C<mocked_state>
+
+Returns true if the sub the object refers to is currently mocked, and false if
+not.
+
 =head2 C<name>
 
 Returns the full name of the sub being mocked, as entered into C<mock()>.
@@ -335,7 +350,6 @@ To work around this and have the side_effect run but still get the
 return_value thereafter, write your cref to evaluate undef as the last thing
 it does: C<sub { ...; undef; }>.
 
-
 =head2 C<return_value>
 
 Add (or change, delete) the mocked sub's return value after instantiation.
@@ -348,6 +362,19 @@ with C<called()> and C<called_count()> back to undef/false.
 
 =head1 NOTES
 
+This module has a backwards parent-child relationship. To use, you create a
+mock object using L<PARENT MOCK OBJECT METHODS> C<new> and C<mock> methods,
+thereafter, you use the returned mocked sub object L<METHODS> to perform the
+work.
+
+The parent mock object retains certain information and statistics of the child
+mocked objects (and the subs themselves).
+
+To mock CORE::GLOBAL functions, you *must* initiate within a C<BEGIN> block
+(see C<SYNOPSIS> for details). It is important that if you mock a CORE sub,
+it can't and won't be returned to its original state until after the entire
+program process tree exists. Period.
+
 I didn't make this a C<Test::> module (although it started that way) because
 I can see more uses than placing it into that category.
 
@@ -370,6 +397,8 @@ Travis-CI: L<https://travis-ci.org/stevieb9/mock-sub>
 
 CPAN Testers: L<http://matrix.cpantesters.org/?dist=Mock-Sub>
 
+Coverage: L<https://coveralls.io/github/stevieb9/mock-sub>
+
 =head1 SUPPORT
 
 You can find documentation for this module with the perldoc command.
@@ -382,7 +411,7 @@ Python's MagicMock module.
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2015 Steve Bertrand.
+Copyright 2016 Steve Bertrand.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
